<%@ page contentType="text/javascript" %>
/**
 * Validates that the date is before the specified validation date
 * Receives the rule definition and its scope.
 * scope.$parent will contain the form to enable watch and access to other fields in the same step
 * ctrls will be an array of controllers, first the ffFormController and then the ngModel of the associated input
 *
 * @param {object} rule current rule definition
 * @param {object} scope current input scope
 * @param {object} el current html element
 * @param {array} attr attributes of the html element
 * @param {array} ctrls ctrls[0] = FFFormController ctrls[1]=ngModel
 */

ff.validationRules.dateBefore = function (rule, scope, el, attr, ctrls) {

    'use strict';
    var minDate = moment('0000', 'YYYY');
    var date = moment(rule.datebefore);
    var formController = ctrls[0];
    return function(modelValue, viewValue){
        //Only return true when value is null, undefined means the value is an invalid date.
        if (viewValue === null) {
            return true;
        }
        if (!formController.parseDate(viewValue)) {
            return false;
        }
        //Validate that the entered date is before the specified date
        var selectedDate = moment(viewValue);
        return selectedDate.isValid() && selectedDate.isBefore(date, 'day') && selectedDate.isAfter(minDate, 'year');
    };
};