<%@ page contentType="text/javascript" %>
/**
 * Validates the file type, file name and file size.
 * Receives the rule definition and its scope.
 * scope.$parent will contain the form to enable watch and access to other fields in the same step
 * ctrls will be an array of controllers, first the ffFormController and then the ngModel of the associated input
 *
 * @param {object} rule current rule definition
 * @param {object} scope current input scope
 * @param {object} el current html element
 * @param {array} attr attributes of the html element
 * @param {array} ctrls ctrls[0] = FFFormController ctrls[1]=ngModel
 */

ff.validationRules.file = function (rule, scope, el, attr, ctrls) {
    'use strict';

    var maxFileSize;
    var conversion;
    if('unittype' in rule) {
        for (var i in rule.unittype) {
            if(rule.unittype[i].selected) {
                switch(rule.unittype[i].value) {

                    case 'kb' :
                            conversion = '*1024';
                            break;
                    case 'mb' :
                            conversion = '*1024*1024';
                            break;
                    case 'gb' :
                            conversion = '*1024*1024*1024';
                            break;
                }
                maxFileSize = eval(rule.filesize + conversion);
            }
        }
    } else {
        maxFileSize = eval(rule.filesize.toUpperCase().replace('GB','*1024*1024*1024').replace('MB','*1024*1024').replace('KB','*1024'));
    }
    return function(modelValue, viewValue){
        //verify if the file upload is initialized and if so, does it have a value.
        if (viewValue !== null && (typeof viewValue === "string" || _.isEmpty(viewValue))) {
            return true;
        }
        if (viewValue) {
            var currentFileType = viewValue[0].type;
            var currentFileSize = viewValue[0].size;
            if (currentFileSize == 0 || currentFileSize > maxFileSize)
                return false;
            var isValid = false;
            for (var k in rule.filetype){
                 if(rule.filetype[k].selected) {
                     try {
                         var fileTypeRegExp = new RegExp(rule.filetype[k].value, 'i');
                         isValid = fileTypeRegExp.test(currentFileType);
                         if (isValid) {
                             break;
                         }
                     }
                     catch(err){
                         console.error('filetype error: ' + rule.filetype[k].value + ' for file type: ' + currentFileType);
                     }
                 }

            }
            return isValid;
        }
        return true;
    };
};
