<%@ page contentType="text/javascript" %>
/**
 * Validates whether the input has the defined amount of selections
 * Receives the rule definition and its scope.
 * scope.$parent will contain the form to enable watch and access to other fields in the same step
 * ctrls will be an array of controllers, first the ffFormController and then the ngModel of the associated input
 *
 * @param {object} rule current rule definition
 * @param {object} scope current input scope
 * @param {object} el current html element
 * @param {array} attr attributes of the html element
 * @param {array} ctrls ctrls[0] = FFFormController ctrls[1]=ngModel
 */

ff.validationRules.numberOfSelection = function(rule, scope, el, attr, ctrls) {
    'use strict';

    var selectionMin = parseInt(rule.selectionmin);
    var selectionMax = parseInt(rule.selectionmax);
    return function(modelValue, viewValue) {
        if(!_.isUndefined(viewValue) && !_.isEmpty(viewValue)) {
            var selections = viewValue.length;
            var validity = true;
            if (!_.isUndefined(selectionMin) && !_.isUndefined(selectionMax)) {
                if (selectionMin === selectionMax && selections !== selectionMin) {
                    validity = false;
                }
                else if (selections < selectionMin || selections > selectionMax) {
                    // Not valid
                    validity = false;
                }
            } else if (!_.isUndefined(selectionMin)) {
                if (selectionMin < selections) {
                    validity = false;
                }
            } else if (!_.isUndefined(selectionMax)) {
                if (selectionMax > selections) {
                    validity = false;
                }
            }
            return validity;
        }
        return false;
    };
};