//Result model for collections
var Result = Backbone.Model.extend({
    initialize: function() {
        this.rendererTemplates = {};
        var that               = this;
        _.each(_resultsRenderer, function(renderer, key, list) {
            that.rendererTemplates[key] = _.template(renderer);
        });
    },
    getRow    : function(labels, site, metadataMode) {
        var jsonModel = this.toJSON();
        var row;
        if (metadataMode) {
            var results = [];
            _.each(labels.slice(3, labels.length), function(labelObject) {
                var result = _.find(jsonModel.results, function (result) {
                    return result.htmlId == labelObject.htmlId;
                });
                if (result) {
                    try {
                        var parsed = JSON.parse(result.values.toString());
                        for (var i in parsed) {
                            if (typeof parsed[i] === 'string') {
                                parsed[i] = _.escape(parsed[i]);
                            }
                        }
                        result.values = JSON.stringify(parsed);
                    } catch (err) {
                        if (moment(result.values.toString(),moment.ISO_8601,true).isValid()) {
                            result.values = moment(result.values.toString()).format('DD/MM/YYYY');
                        } else {
                            result.values = _.escape(result.values.toString());
                        }
                    }
                    results.push(result);
                }
            });
            jsonModel.results = results;
            row = [jsonModel.username,
                '<a onclick=\'displayData(' + JSON.stringify(jsonModel) + ')\'>' + moment(jsonModel.submissionDate).format('LLL') + '</a>',
                '<a target=\'#\' href=\'' + jsonModel.origin + '\'>' + jsonModel.origin.split(site)[1] + '</a>', jsonModel.remoteAddress];
        } else {
            row      = [jsonModel.username,
                moment(jsonModel.submissionDate).format('LLL'),
                '<a target=\'#\' href=\'' + jsonModel.origin + '\'>' + jsonModel.origin.split(site)[1] + '</a>',
                jsonModel.remoteAddress
            ];
            var that = this;
            _.each(labels.slice(4, labels.length), function(labelObject) {
                var result = _.find(jsonModel.results, function(result) {
                    return result.htmlId == labelObject.htmlId;
                });
                if (result) {
                    var rendered;
                    try {
                        var parsed       = JSON.parse(result.values.toString());
                        var rendererName = (_.isObject(parsed) && _.has(parsed, 'rendererName')) ? parsed.rendererName : (_.isObject(parsed.value) && _.has(parsed.value, 'rendererName')) ? parsed.value.rendererName : undefined;
                        if (rendererName !== undefined && that.rendererTemplates[rendererName] !== undefined) {
                            for (var i in parsed ) {
                                if (typeof parsed[i] === 'string') {
                                    parsed[i] = _.escape(parsed[i]);
                                }
                            }
                            rendered = that.rendererTemplates[rendererName]({data: parsed});
                        } else if (_.isBoolean(parsed)) {
                            rendered = '<span><i class="fa fa-2x ' + (parsed ? 'fa-toggle-on' : 'fa-toggle-off') + '"></i></span>';
                        } else {
                            rendered = '<code>' + JSON.stringify(parsed, null, '<br/><br/>') + '</code>';
                        }
                    } catch (err) {
                        if (moment(result.values.toString(),moment.ISO_8601,true).isValid()) {
                            rendered = moment(result.values.toString()).format('DD/MM/YYYY');
                        } else {
                            rendered = _.escape(result.values.toString());
                        }
                    }
                    row.push(rendered);
                } else {
                    row.push('N/A');
                }
            });
        }
        return row;
    }
});

//Results Collection (Containing results models)
var ResultsCollection = Backbone.Collection.extend({
    model  : Result,
    toDate : new Date().getTime(),
    url    : function() {
        this.fromDate = moment(this.fromDate).hour(0);
        this.fromDate = moment(this.fromDate).minute(0);
        this.toDate   = moment(this.toDate).hour(23);
        this.toDate   = moment(this.toDate).minute(59);

        return getFetchURL({
            formId       : this.formId,
            apiMethodName: 'results',
            fromDate     : this.fromDate,
            toDate       : this.toDate
        });
    },
    parse  : function(response) {
        return response.data;
    },
    getRows: function(labels, site, metadataMode) {
        var rows = [];
        _.each(this.models, function(model, index) {
            var row = model.getRow(labels, site, metadataMode);
            rows.push(row);
        });
        return rows;
    }
});

//Result Pageable Collection
var PageableResults = Backbone.PageableCollection.extend({
    model      : Result,
    url        : function() {
        this.fromDate = moment(this.fromDate).hour(0);
        this.fromDate = moment(this.fromDate).minute(0);
        this.toDate   = moment(this.toDate).hour(23);
        this.toDate   = moment(this.toDate).minute(59);
        return getFetchURL({
            formId       : this.formId,
            apiMethodName: 'results',
            fromDate     : this.fromDate,
            toDate       : this.toDate
        });
    },
    state      : {
        pageSize : 500,
        sortKey  : 'updated',
        order    : 1,
        firstPage: 0
    },
    queryParams: {
        totalPages  : null,
        totalRecords: null
    },
    parse      : function(response) {
        return response.data;

    },
    getRows    : function(labels, site) {
        var rows = [];
        _.each(this.models, function(model, index) {
            var row = model.getRow(labels, site);
            rows.push(row);
        });
        return rows;
    },
    comparator : function(model) {
        return model.submissionDate;
    }
});
