package org.jahia.modules.formfactory.dsl

import groovy.json.JsonBuilder
import org.jahia.api.Constants
import org.jahia.data.templates.JahiaTemplatesPackage
import org.jahia.services.content.*
import org.jahia.services.content.nodetypes.ExtendedNodeType
import org.jahia.services.templates.TemplatePackageRegistry
import org.jahia.services.usermanager.JahiaUserManagerService
import org.jahia.settings.SettingsBean
import org.slf4j.Logger
import org.slf4j.LoggerFactory

import javax.jcr.RepositoryException

class LogicHandler implements DSLHandler {
    private static final Logger logger = LoggerFactory.getLogger(LogicHandler.class);
    def JCRTemplate jcrTemplate;
    def TemplatePackageRegistry packageRegistry
    def SettingsBean settingsBean

    def logic(@DelegatesTo(FormFactoryWizardParser) Closure cl) {
        parseDeclaration(cl)
    }

    def parseDeclaration(Closure cl) {
        def logicType = new FormFactoryWizardParser()
        def code = cl.rehydrate(logicType, this, this)
        code.resolveStrategy = Closure.DELEGATE_FIRST
        code()
        logger.info("Declared logic structure: " + logicType.toString());
        def result = logicType.contentMap
        createWizard result, cl.owner.currentPackage, cl.owner.currentNodeType
    }

    def registerProperties(wizardEntry, value) {
        if (value instanceof String) {
            wizardEntry.setProperty("jsonValue", value)
        } else if (value instanceof Map) {
            assert value["values"] != null

            def values = value["values"]
            if (values instanceof List) {
                def jsonValue = []
                values.eachWithIndex { label, idx ->
                    jsonValue[idx] = [label: label, value: label.toString().toLowerCase(), selected: (idx == 0)]
                }
                def json = new JsonBuilder()
                json jsonValue
                wizardEntry.setProperty("jsonValue", json.toString())
            } else if (values instanceof Map) {
                def jsonValue = []
                values.eachWithIndex { k, l, idx ->
                    jsonValue[idx] = [key: k, value: l]
                }
                def json = new JsonBuilder()
                json jsonValue
                wizardEntry.setProperty("jsonValue", json.toString())
            }
        } else if (value instanceof List) {
            wizardEntry.setProperty("jsonValue", value[1])
        }
    }

    def createWizard(Map<String, Object> map, JahiaTemplatesPackage currentPackage, ExtendedNodeType currentNodeType) {
        assert map.containsKey("label")
        assert map.containsKey("types")
        jcrTemplate.doExecuteWithSystemSessionAsUser(JahiaUserManagerService.instance.lookupRootUser().jahiaUser, Constants.EDIT_WORKSPACE, Locale.ENGLISH, new JCRCallback() {
            @Override
            Object doInJCR(JCRSessionWrapper jcrSessionWrapper) throws RepositoryException {
                JCRNodeWrapper bundleRootNode = jcrSessionWrapper.getNode(currentPackage.rootFolderPath + "/" + currentPackage.getVersion().toString())
                def rNodeWrapper = bundleRootNode.getNode("templates").getNode("contents")
                if (!rNodeWrapper.hasNode("form-factory-logics")) {
                    rNodeWrapper.addNode("form-factory-logics","jnt:contentFolder")
                }
                def logicsRootNode = rNodeWrapper.getNode("form-factory-logics")
                def nodeName = JCRContentUtils.generateNodeName(map["label"], 32)

                if (logicsRootNode.hasNode(nodeName)) {
                    //Reintroduce check for development if necessary
                    /*if (settingsBean.isDevelopmentMode()) {
                        logicsRootNode.getNode(nodeName).remove()
                        jcrSessionWrapper.save()
                    } else {
                        return null;
                    }*/
                    return null;
                }

                def node = logicsRootNode.addNode(nodeName, currentNodeType.name)
                node.setProperty(Constants.JCR_TITLE, map["label"])
                map.remove("label")

                def types
                if (map.get("types") instanceof List) {
                    types = (List<String>) map.get("types")
                } else {
                    types = [map.get("types")]
                }
                node.setProperty("types", types.toArray(new String[types.size()]))
                map.remove("types")

                if (map.containsKey("wizard")) {
                    node.setProperty("wizard", map["wizard"])
                    map.remove("wizard")
                }
                
                if (map.containsKey("properties")) {
                    def properties = map["properties"]
                    properties.each { key, value ->
                        def wizardEntry = node.addNode(key, "fcnt:definitionOptions")
                        registerProperties(wizardEntry, value)
                    }
                    map.remove("properties")
                }

                if (map.containsKey("propertiesI18n")) {
                    def propertiesI18n = map["propertiesI18n"]
                    propertiesI18n.each { key, value ->
                        def wizardEntry = node.addNode(key, "fcnt:definitionOptionsTranslatable")
                        registerProperties(wizardEntry, value)
                    }
                    map.remove("propertiesI18n")
                }

                jcrSessionWrapper.save()
                return null
            }
        })
    }
}

