package org.jahia.modules.formfactory.dsl

import groovy.json.JsonBuilder
import org.jahia.api.Constants
import org.jahia.data.templates.JahiaTemplatesPackage
import org.jahia.services.content.*
import org.jahia.services.content.nodetypes.ExtendedNodeType
import org.jahia.services.templates.TemplatePackageRegistry
import org.jahia.services.usermanager.JahiaUserManagerService
import org.jahia.settings.SettingsBean
import org.slf4j.Logger
import org.slf4j.LoggerFactory

import javax.jcr.RepositoryException

class RendererHandler implements DSLHandler {
    private static final Logger logger = LoggerFactory.getLogger(RendererHandler.class);
    def JCRTemplate jcrTemplate;
    def TemplatePackageRegistry packageRegistry
    def SettingsBean settingsBean

    def renderer(@DelegatesTo(FormFactoryWizardParser) Closure cl) {
        parseDeclaration(cl)
    }

    def parseDeclaration(Closure cl) {
        def rendererType = new FormFactoryWizardParser()
        def code = cl.rehydrate(rendererType, this, this)
        code.resolveStrategy = Closure.DELEGATE_FIRST
        code()
        logger.info("Declared renderer structure: " + rendererType.toString());
        def result = rendererType.contentMap
        createWizard result, cl.owner.currentPackage, cl.owner.currentNodeType
    }

    def createWizard(Map<String, Object> map, JahiaTemplatesPackage currentPackage, ExtendedNodeType currentNodeType) {
        assert map.containsKey("label")
        assert map.containsKey("name")
        jcrTemplate.doExecuteWithSystemSessionAsUser(JahiaUserManagerService.instance.lookupRootUser().jahiaUser, Constants.EDIT_WORKSPACE, Locale.ENGLISH, new JCRCallback() {
            @Override
            Object doInJCR(JCRSessionWrapper jcrSessionWrapper) throws RepositoryException {
                JCRNodeWrapper bundleRootNode = jcrSessionWrapper.getNode(currentPackage.rootFolderPath + "/" + currentPackage.getVersion().toString())
                def rNodeWrapper = bundleRootNode.getNode("templates").getNode("contents")
                if (!rNodeWrapper.hasNode("form-factory-renderers")) {
                    rNodeWrapper.addNode("form-factory-renderers", "jnt:contentFolder")
                }
                def rendererRootNode = rNodeWrapper.getNode("form-factory-renderers")
                def nodeName = JCRContentUtils.generateNodeName(map["label"], 32)

                if (rendererRootNode.hasNode(nodeName)) {
                    //Reintroduce check for development if necessary
                    /*if (settingsBean.isDevelopmentMode()) {
                        rendererRootNode.getNode(nodeName).remove()
                        jcrSessionWrapper.save()
                    } else {
                        return null;
                    }*/
                    return null;
                }

                def node = rendererRootNode.addNode(nodeName, currentNodeType.name)
                node.setProperty(Constants.JCR_TITLE, map["label"])
                map.remove("label")
                node.setProperty("rendererName", map["name"])
                map.remove("name")

                jcrSessionWrapper.save()
                return null
            }
        })
    }
}

